/*
 * Copyright 2002-2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.fzm.mall.midea.pay.utils;

import org.apache.commons.lang3.StringUtils;

import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.StandardCharsets;
import java.security.GeneralSecurityException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * Miscellaneous methods for calculating digests.
 * <p>Mainly for internal use within the framework; consider
 * <a href="http://commons.apache.org/codec/">Apache Commons Codec</a> for a
 * more comprehensive suite of digest utilities.
 *
 * @author Arjen Poutsma
 * @see org.apache.commons.codec.digest.DigestUtils
 * @since 3.0
 */
public abstract class DigestUtils{

    private static final String MD5_ALGORITHM_NAME = "MD5";

    private static final char[] HEX_CHARS = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e',
        'f' };

    /**
     * Calculate the MD5 digest of the given bytes.
     *
     * @param bytes the bytes to calculate the digest over
     * @return the digest
     */
    public static byte[] md5Digest(byte[] bytes) {

        return digest(MD5_ALGORITHM_NAME, bytes);
    }

    /**
     * Return a hexadecimal string representation of the MD5 digest of the given
     * bytes.
     *
     * @param bytes the bytes to calculate the digest over
     * @return a hexadecimal digest string
     */
    public static String md5DigestAsHex(byte[] bytes) {

        return digestAsHexString(MD5_ALGORITHM_NAME, bytes);
    }

    public static String getMD5String(String param) {

        if (StringUtils.isEmpty(param))
            return param;
        byte[] result = digest(MD5_ALGORITHM_NAME, param.getBytes(StandardCharsets.UTF_8));
        String str = new String(encodeHex(result));
        return str.toUpperCase();
    }

    /**
     * Append a hexadecimal string representation of the MD5 digest of the given
     * bytes to the given {@link StringBuilder}.
     *
     * @param bytes   the bytes to calculate the digest over
     * @param builder the string builder to append the digest to
     * @return the given string builder
     */
    public static StringBuilder appendMd5DigestAsHex(byte[] bytes, StringBuilder builder) {

        return appendDigestAsHex(MD5_ALGORITHM_NAME, bytes, builder);
    }

    /**
     * Creates a new {@link MessageDigest} with the given algorithm. Necessary
     * because {@code MessageDigest} is not thread-safe.
     */
    private static MessageDigest getDigest(String algorithm) {

        try {
            return MessageDigest.getInstance(algorithm);
        } catch (NoSuchAlgorithmException ex) {
            throw new IllegalStateException("Could not find MessageDigest with algorithm \"" + algorithm + "\"", ex);
        }
    }

    private static byte[] digest(String algorithm, byte[] bytes) {

        return getDigest(algorithm).digest(bytes);
    }

    private static String digestAsHexString(String algorithm, byte[] bytes) {

        char[] hexDigest = digestAsHexChars(algorithm, bytes);
        return new String(hexDigest);
    }

    private static StringBuilder appendDigestAsHex(String algorithm, byte[] bytes, StringBuilder builder) {

        char[] hexDigest = digestAsHexChars(algorithm, bytes);
        return builder.append(hexDigest);
    }

    public static String aesEncrypt(String plaintext, String key) {

        byte[] aes = aes(plaintext.getBytes(StandardCharsets.UTF_8), key.getBytes(StandardCharsets.UTF_8),
            Cipher.ENCRYPT_MODE);
        return new String(encodeHex1(aes));

    }

    private static char[] digestAsHexChars(String algorithm, byte[] bytes) {

        byte[] digest = digest(algorithm, bytes);
        return encodeHex(digest);
    }

    public static char[] encodeHex(byte[] bytes) {

        char chars[] = new char[32];
        for (int i = 0; i < chars.length; i = i + 2) {
            byte b = bytes[i / 2];
            chars[i] = HEX_CHARS[(b >>> 0x4) & 0xf];
            chars[i + 1] = HEX_CHARS[b & 0xf];
        }
        return chars;
    }
    protected static char[] encodeHex1(final byte[] data) {
        final int l = data.length;
        final char[] out = new char[l << 1];
        // two characters form the hex value.
        for (int i = 0, j = 0; i < l; i++) {
            out[j++] = HEX_CHARS[(0xF0 & data[i]) >>> 4];
            out[j++] = HEX_CHARS[0x0F & data[i]];
        }
        return out;
    }

    /**
     * 字符串MD5加密
     *
     * @throws Exception
     */
    public static String encryptMD5(String strInput) {

        return encryptMD5(strInput, "UTF-8");
    }

    public static String encryptMD5(String strInput, String encoder) {

        try {
            StringBuffer buf = null;
            MessageDigest md;
            md = MessageDigest.getInstance("MD5");
            md.update(strInput.getBytes(encoder));
            byte b[] = md.digest();
            buf = new StringBuffer(b.length * 2);
            for (int i = 0; i < b.length; i++) {
                if (((int) b[i] & 0xff) < 0x10) { // & 0xff转换无符号整型
                    buf.append("0");
                }
                // buf.append(Long.toString((int) b[i] & 0xff,
                // 16));//转换16进制,下方法同
                buf.append(Long.toHexString((int) b[i] & 0xff));
            }

            String result = buf.toString();
            return result;
        } catch (Exception e) {
            return strInput;
        }
    }

    public static byte[] aesEncrypt(byte[] input, byte[] key) {

        return aes(input, key, Cipher.ENCRYPT_MODE);
    }

    private static byte[] aes(byte[] input, byte[] key, int mode) {

        try {
            SecretKey secretKey = new SecretKeySpec(key, "AES");
            Cipher cipher = Cipher.getInstance("AES");
            cipher.init(mode, secretKey);
            return cipher.doFinal(input);
        } catch (GeneralSecurityException e) {
            e.printStackTrace();
        }
        return null;
    }

}
